function FourierMATLAB
% Vypocet koeficientu Fourierovy rady numerickym integrovanim
% clc; % mazani Commmand Window
close all; % zavreni vsech oken


%%%%%%% Zadani prikladu a numericke metody integrace %%%%%%%%
priklad=1; % prepinani prikladu
integr=1;  % prepinani metody numerickeho integrovani 1/2 == Lichobeznik/Simpson's
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% zadani parametru funkce
if priklad==1
    %%% Priklad 1
    omega=1; % uhlova rychlost [rad/s]
    T=2*pi/omega; % perioda [s]
elseif priklad==2
    %%% Priklad 2 - obdelnikovy impuls
    T=2; % perioda [s]
    omega=2*pi/T; % uhlova rychlost [rad/s]
end

deltaT=T/100; % integracni krok
t=[0:deltaT:T]'; % diskretizace

if priklad==1
    %%% Pr.1 - zadani funkce f=sin^2(omega*t)
    f=sin(omega*t).^2;
elseif priklad==2
    %%% Pr.2 - zadani funkce obdelnikoveho impulsu f=1 pro t=<0,1), f=0 pro t=<1,2>
    f=zeros(length(t),1);
    f(1:((end-1)/2))=ones(length(f(1:((end-1)/2))),1);
end



% vypocet n koeficientu Fourierovy rady
n=10; % pocet koeficientu

% vypocet a0
if integr==1 % Lichobeznikova metoda
    a0=2/T*intTrapezoidal(f,deltaT);
elseif integr==2 % Simsonovo pravidlo
    a0=2/T*intSimpsons(f,deltaT);
end

tol=1e-5; % chyba vypoctu - numericke integrace, "numericka nula"
% vypocet a1,a2...an, b1,b2,...bn
for k=1:n
   if integr==1 % Lichobeznikova metoda
       a(k)=2/T*intTrapezoidal(f.*cos(k*omega*t),deltaT);
       b(k)=2/T*intTrapezoidal(f.*sin(k*omega*t),deltaT);
   elseif integr==2 % Simsonovo pravidlo
       a(k)=2/T*intSimpsons(f.*cos(k*omega*t),deltaT);
       b(k)=2/T*intSimpsons(f.*sin(k*omega*t),deltaT);
   end
end


% tisk koeficientu
fourier=zeros(length(t),1);
if abs(a0)>tol
    fprintf('a0 : %d\n',a0)
    fourier=a0/2*ones(length(t),1);
end
for k=1:n
    if abs(a(k))>tol
        fprintf('a%u : %d\n', k, a(k));
        fourier=fourier+a(k)*cos(k*omega*t);
    end
    if abs(b(k))>tol
        fprintf('b%u : %d\n', k, b(k));
        fourier=fourier+b(k)*sin(k*omega*t);
    end
end


% vykresleni reseni
figure
plot(t,f,t,fourier)
xlabel('t')
legend('f(t)','fourier(t)')


% vykresleni absolutni chyby
error=abs(fourier-f);
figure
plot(t,error)
xlabel('t')
legend('absError')


errorL2norm=sqrt(intTrapezoidal(error.*error,deltaT));
% errorNorm=norm(error,inf);
fprintf('Chyba aproximace ||f(t)-fourier(t)||L2: %d \n\n', errorL2norm)
end





%%%%%%% POMOCNE FUNKCE - numericke integrovani %%%%%%%%%
function I=intTrapezoidal(f,h)

I=h/2*(f(1)+f(end)+2*sum(f(2:end-1)));

end

function I=intSimpsons(f,h)

I=h/3*(f(1)+f(end)+4*sum(f(2:2:end-1))+2*sum(f(3:2:end-2)));

end
